



//################################################################################################
//##################                                                            ##################
//##################                   Bit manipulation tools                   ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################




/*
 * Bits: High-efficiency Bit manipulation tools.
 *
 * Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 * Contact: varkhan@varkhan.net
 * Website: http://www.varkhan.net/
 * Project: http://www.varkhan.net/software/VCom
 */
#ifndef _VCOM_BITS_H_
#define _VCOM_BITS_H_







//################################################################################################
//###
//###  Bit counts and positions
//###


// Function ctbit
// ==============
// 
// Computes the number of set bits.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _the number of bytes set in the argument

unsigned long vcom_bctbit(unsigned char bits);
unsigned long vcom_lctbit(unsigned long bits);
unsigned long vcom_vctbit(void * bits, unsigned long size);


// Function lobit
// ==============
// 
// Computes the number of trailing 0-bits, starting at the least significant (or leftmost) bit position.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _-1            : if the argument is zero
//            _the number of 0-bits from the least significant, or leftmost, to the first 1-bit of the argument
// 
//   Note:
//            The value returned for every odd value (for which the least significant bit is set) is 0.
//            The value returned for values of the type 2^k, where k>=0, is k.

unsigned long vcom_blobit(unsigned char bits);
unsigned long vcom_llobit(unsigned long bits);
unsigned long vcom_vlobit(void * bits, unsigned long size);

unsigned long vcom_vlobit1(void * bits, unsigned long size);


// Function hibit
// ==============
// Computes the number of leading 0-bits, starting at the most significant (or rightmost) bit position.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _-1            : if the argument is zero
//            _the number of 0-bits from the most significant, or rightmost, to the first 1-bit of the argument
// 
//   Note:
//            The value returned for values of the type 2^k, where k>=0, is k.
//            More generally, the value returned is the biggest integer k>=0 such that 2^k is smaller than the argument.

unsigned long vcom_bhibit(unsigned char bits);
unsigned long vcom_lhibit(unsigned long bits);
unsigned long vcom_vhibit(void * bits, unsigned long size);

unsigned long vcom_vhibit1(void * bits, unsigned long size);




//###
//###  END
//###
//################################################################################################
#endif // _VCOM_BITS_H_
