



//################################################################################################
//##################                                                            ##################
//##################              Indexed Hash Set Datastructure                ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################
///





// ISet: High-efficiency Indexed Hash Set object model.

// Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan


// This library is free software;  you can redistribute it and/or modify it
// under  the terms of the  GNU Lesser General Public License  as published
// by the  Free Software Foundation;  either version 2.1 of the License, or
// (at your option) any later version.

// This  library  is  distributed  in  the hope that it will be useful, but
// WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
// General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this library;  if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


// Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
// Contact: varkhan@varkhan.net
// Website: http://www.varkhan.net/
// Project: http://www.varkhan.net/software/VCom
///
#ifndef _VCOM_ISET_H_
#define _VCOM_ISET_H_
// Indexed hash set







// The prototype of mixing functions
typedef unsigned long (* vcom_iset_mixr)(void * key);




// The prototype of comparator functions
typedef unsigned char (* vcom_iset_cmpr)(void * key1, void * key2);




// The prototype of visitor callbacks
typedef unsigned long (* vcom_iset_visitor)(void * key, unsigned long idx, void * dat);




//################################################################################################
//###
//###  Structures and types
//###



// The opaque ISet object definition
typedef struct vcom_iset_dlopen_s * vcom_iset_dlopen_t;


//################################################################################################
//###
//###  Object lifecycle
//###


// Hash Set constructor
vcom_iset_dlopen_t vcom_iset_dlopen_new(unsigned long blen, unsigned long capa, vcom_iset_mixr mixf, vcom_iset_cmpr cmpf, float lfct, float gfct);

// Hash Set duplicator
vcom_iset_dlopen_t vcom_iset_dlopen_cpy(vcom_iset_dlopen_t iset);

// Hash Set destructor
void vcom_iset_dlopen_del(vcom_iset_dlopen_t iset);



//################################################################################################
//###
//###  Parameters accessors
//###


// Hash Set size
unsigned long vcom_iset_dlopen_size(vcom_iset_dlopen_t iset);

// Hash Set head
unsigned long vcom_iset_dlopen_head(vcom_iset_dlopen_t iset);

// Hash Set free idx
unsigned long vcom_iset_dlopen_free(vcom_iset_dlopen_t iset);

// Hash Set capacity
unsigned long vcom_iset_dlopen_capa(vcom_iset_dlopen_t iset);

// Get default idx
unsigned long vcom_iset_dlopen_getdef(vcom_iset_dlopen_t iset);

// Set default idx
void vcom_iset_dlopen_setdef(vcom_iset_dlopen_t iset, unsigned long val);



//################################################################################################
//###
//###  Bulk operations
//###


// Clears all elements
void vcom_iset_dlopen_clr(vcom_iset_dlopen_t iset);

// Release unused ressources
void vcom_iset_dlopen_adj(vcom_iset_dlopen_t iset, unsigned long capa);



//################################################################################################
//###
//###  Elementary operations
//###


// Hash Set search
unsigned long vcom_iset_dlopen_idx(vcom_iset_dlopen_t iset, void * key);

// Hash Set test
unsigned char vcom_iset_dlopen_has(vcom_iset_dlopen_t iset, unsigned long idx);

// Hash Set retrieve
void * vcom_iset_dlopen_get(vcom_iset_dlopen_t iset, unsigned long idx);

// Hash Set update
unsigned long vcom_iset_dlopen_put(vcom_iset_dlopen_t iset, void * key);

// Hash Set remove
void * * vcom_iset_dlopen_rem(vcom_iset_dlopen_t iset, unsigned long idx, void * * key);

// Apply callback to all entries, in slot order
unsigned long vcom_iset_dlopen_visit(vcom_iset_dlopen_t iset, vcom_iset_visitor visitor, void * dat);




//###
//###  END
//###
//################################################################################################
#endif // _VCOM_ISET_H_
