



//################################################################################################
//##################                                                            ##################
//##################                Indexed List Datastructure                  ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################
///





// List: High-efficiency Indexed List object model.

// Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan


// This library is free software;  you can redistribute it and/or modify it
// under  the terms of the  GNU Lesser General Public License  as published
// by the  Free Software Foundation;  either version 2.1 of the License, or
// (at your option) any later version.

// This  library  is  distributed  in  the hope that it will be useful, but
// WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
// General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this library;  if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


// Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
// Contact: varkhan@varkhan.net
// Website: http://www.varkhan.net/
// Project: http://www.varkhan.net/software/VCom
///
#ifndef _VCOM_LIST_H_
#define _VCOM_LIST_H_
// Sparse list
// The prototype of visitor callbacks
typedef unsigned long (* vcom_list_visitor)(unsigned long idx, void * * val, void * dat);
//################################################################################################
//###
//###  Structures and types
//###
// The opaque List object definition
typedef struct vcom_list_sparse_s * vcom_list_sparse_t;
//################################################################################################
//###
//###  Object lifecycle
//###
// Builds a new list in memory
vcom_list_sparse_t vcom_list_sparse_new(unsigned long blen, unsigned long capa);
// Copies a list in another memory segment
vcom_list_sparse_t vcom_list_sparse_cpy(vcom_list_sparse_t list);
// Deletes a list from memory
void vcom_list_sparse_del(vcom_list_sparse_t list);
//################################################################################################
//###
//###  Parameters accessors
//###
// The number of elements in the list
unsigned long vcom_list_sparse_size(vcom_list_sparse_t list);
// The head index (highest unused index) in the list
unsigned long vcom_list_sparse_head(vcom_list_sparse_t list);
// The tail index (lowest unused index) in the list
unsigned long vcom_list_sparse_tail(vcom_list_sparse_t list);
// The capacity (total numer of slots) in the list
unsigned long vcom_list_sparse_capa(vcom_list_sparse_t list);
//################################################################################################
//###
//###  Bulk operations
//###
// Clears all elements
void vcom_list_sparse_clr(vcom_list_sparse_t list);
// Release unused ressources
void vcom_list_sparse_adj(vcom_list_sparse_t list, unsigned long capa);
//################################################################################################
//###
//###  Elementary operations
//###
// Tests the presence of an element in a slot
unsigned char vcom_list_sparse_has(vcom_list_sparse_t list, unsigned long idx);
// Retries an element from a slot
void * * vcom_list_sparse_get(vcom_list_sparse_t list, unsigned long idx);
// Updates the contents of a slot
void * * vcom_list_sparse_set(vcom_list_sparse_t list, unsigned long idx, void * val);
// Removes (empties) a slot from its element
void * * vcom_list_sparse_rem(vcom_list_sparse_t list, unsigned long idx, void * * val);
// Apply a callback to all entries, in slot order
unsigned long vcom_list_sparse_visit(vcom_list_sparse_t list, vcom_list_visitor visitor, void * dat);
//###
//###  END
//###
//################################################################################################
// Block list
//################################################################################################
//###
//###  Structures and types
//###
// The opaque List object definition
typedef struct vcom_list_block_s * vcom_list_block_t;
//################################################################################################
//###
//###  Object lifecycle
//###
// Builds a new list in memory
vcom_list_block_t vcom_list_block_new(unsigned long blen, unsigned long capa);
// Copies a list in another memory segment
vcom_list_block_t vcom_list_block_cpy(vcom_list_block_t list);
// Deletes a list from memory
void vcom_list_block_del(vcom_list_block_t list);
//################################################################################################
//###
//###  Parameters accessors
//###
// The number of elements in the list
unsigned long vcom_list_block_size(vcom_list_block_t list);
// The head index (highest unused index) in the list
unsigned long vcom_list_block_head(vcom_list_block_t list);
// The tail index (lowest unused index) in the list
unsigned long vcom_list_block_tail(vcom_list_block_t list);
// The capacity (total numer of slots) in the list
unsigned long vcom_list_block_capa(vcom_list_block_t list);
//################################################################################################
//###
//###  Bulk operations
//###
// Clears all elements
void vcom_list_block_clr(vcom_list_block_t list);
// Release unused ressources
void vcom_list_block_adj(vcom_list_block_t list, unsigned long capa);
//################################################################################################
//###
//###  Elementary operations
//###
// Tests the presence of an element in a slot
unsigned char vcom_list_block_has(vcom_list_block_t list, unsigned long idx);
// Retries an element from a slot
void * * vcom_list_block_get(vcom_list_block_t list, unsigned long idx);
// Updates the contents of a slot
void * * vcom_list_block_set(vcom_list_block_t list, unsigned long idx, void * val);
// Removes (empties) a slot from its element
void * * vcom_list_block_rem(vcom_list_block_t list, unsigned long idx, void * * val);
// Apply a callback to all entries, in slot order
unsigned long vcom_list_block_visit(vcom_list_block_t list, vcom_list_visitor visitor, void * dat);
//###
//###  END
//###
//################################################################################################
#endif // _VCOM_LIST_H_
