#ifndef __BITS__HEADER__
#define __BITS__HEADER__
//################################################################################################
//##################                                                            ##################
//##################                   Bit manipulation tools                   ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################
#endif // __BITS__HEADER__

#ifndef __BITS__LICENCE__
#define __BITS__LICENCE__
/*
 * Bits: High-efficiency Bit manipulation tools.
 *
 * Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 * Contact: varkhan@varkhan.net
 * Website: http://www.varkhan.net/
 * Project: http://www.varkhan.net/software/VCom
 */
#endif


#ifndef __BITS__BODY__
#define __BITS__BODY__




//################################################################################################
//###
//###  Bit counts and positions
//###


// Function ctbit
// ==============
// 
// Computes the number of set bits.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _the number of bytes set in the argument

_Bits_size_t _Bits_b(ctbit)(_Bits_byte_t bits);
_Bits_size_t _Bits_l(ctbit)(_Bits_long_t bits);
_Bits_size_t _Bits_v(ctbit)(void * bits, _Bits_size_t size);


// Function lobit
// ==============
// 
// Computes the number of trailing 0-bits, starting at the least significant (or leftmost) bit position.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _-1            : if the argument is zero
//            _the number of 0-bits from the least significant, or leftmost, to the first 1-bit of the argument
// 
//   Note:
//            The value returned for every odd value (for which the least significant bit is set) is 0.
//            The value returned for values of the type 2^k, where k>=0, is k.

_Bits_size_t _Bits_b(lobit)(_Bits_byte_t bits);
_Bits_size_t _Bits_l(lobit)(_Bits_long_t bits);
_Bits_size_t _Bits_v(lobit)(void * bits, _Bits_size_t size);

_Bits_size_t _Bits_v(lobit1)(void * bits, _Bits_size_t size);


// Function hibit
// ==============
// Computes the number of leading 0-bits, starting at the most significant (or rightmost) bit position.
// 
//   Args:
//            _bits          : a scalar or a pointer to a contiguous memory region
//            _size          : the number of bytes in the buffer
// 
//   Returns:
//            _-1            : if the argument is zero
//            _the number of 0-bits from the most significant, or rightmost, to the first 1-bit of the argument
// 
//   Note:
//            The value returned for values of the type 2^k, where k>=0, is k.
//            More generally, the value returned is the biggest integer k>=0 such that 2^k is smaller than the argument.

_Bits_size_t _Bits_b(hibit)(_Bits_byte_t bits);
_Bits_size_t _Bits_l(hibit)(_Bits_long_t bits);
_Bits_size_t _Bits_v(hibit)(void * bits, _Bits_size_t size);

_Bits_size_t _Bits_v(hibit1)(void * bits, _Bits_size_t size);




//###
//###  END
//###
//################################################################################################
#endif //__BITS__BODY__




