#ifndef __ISET__HEADER__
#define __ISET__HEADER__
//################################################################################################
//##################                                                            ##################
//##################              Indexed Hash Set Datastructure                ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################
///
#endif // __ISET__HEADER__

#ifndef __ISET__LICENCE__
#define __ISET__LICENCE__
//
// ISet: High-efficiency Indexed Hash Set object model.
//
// Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
//
//
// This library is free software;  you can redistribute it and/or modify it
// under  the terms of the  GNU Lesser General Public License  as published
// by the  Free Software Foundation;  either version 2.1 of the License, or
// (at your option) any later version.
//
// This  library  is  distributed  in  the hope that it will be useful, but
// WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
// General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library;  if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//
// Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
// Contact: varkhan@varkhan.net
// Website: http://www.varkhan.net/
// Project: http://www.varkhan.net/software/VCom
///
#endif // __ISET__LICENCE__



#ifndef __ISET__BODY__
#define __ISET__BODY__


#ifndef __ISET__TYPEDEF_ISet_mix_t
#define __ISET__TYPEDEF_ISet_mix_t
// The prototype of mixing functions
typedef _ISet_size_t (* _ISet_mix_t)(_ISet_key_t key);
#endif

#ifndef __ISET__TYPEDEF_ISet_cmp_t
#define __ISET__TYPEDEF_ISet_cmp_t
// The prototype of comparator functions
typedef _ISet_bool_t (* _ISet_cmp_t)(_ISet_key_t key1, _ISet_key_t key2);
#endif

#ifndef __ISET__TYPEDEF_ISet_vis_t
#define __ISET__TYPEDEF_ISet_vis_t
// The prototype of visitor callbacks
typedef _ISet_size_t (* _ISet_vis_t)(_ISet_key_t key, _ISet_idx_t idx, void * dat);
#endif



//################################################################################################
//###
//###  Structures and types
//###



// The opaque ISet object definition
typedef struct _ISet_struct_t * _ISet_obj_t;


//################################################################################################
//###
//###  Object lifecycle
//###


// Hash Set constructor
_ISet_obj_t _ISet_m(new)(_ISet_size_t blen, _ISet_size_t capa, _ISet_mix_t mixf, _ISet_cmp_t cmpf, float lfct, float gfct);

// Hash Set duplicator
_ISet_obj_t _ISet_m(cpy)(_ISet_obj_t iset);

// Hash Set destructor
void _ISet_m(del)(_ISet_obj_t iset);



//################################################################################################
//###
//###  Parameters accessors
//###


// Hash Set size
_ISet_size_t _ISet_m(size)(_ISet_obj_t iset);

// Hash Set head
_ISet_idx_t _ISet_m(head)(_ISet_obj_t iset);

// Hash Set free idx
_ISet_idx_t _ISet_m(free)(_ISet_obj_t iset);

// Hash Set capacity
_ISet_size_t _ISet_m(capa)(_ISet_obj_t iset);

// Get default idx
_ISet_idx_t _ISet_m(getdef)(_ISet_obj_t iset);

// Set default idx
void _ISet_m(setdef)(_ISet_obj_t iset, _ISet_idx_t val);



//################################################################################################
//###
//###  Bulk operations
//###


// Clears all elements
void _ISet_m(clr)(_ISet_obj_t iset);

// Release unused ressources
void _ISet_m(adj)(_ISet_obj_t iset, _ISet_size_t capa);



//################################################################################################
//###
//###  Elementary operations
//###


// Hash Set search
_ISet_idx_t _ISet_m(idx)(_ISet_obj_t iset, _ISet_key_t key);

// Hash Set test
_ISet_bool_t _ISet_m(has)(_ISet_obj_t iset, _ISet_idx_t idx);

// Hash Set retrieve
_ISet_key_t _ISet_m(get)(_ISet_obj_t iset, _ISet_idx_t idx);

// Hash Set update
_ISet_idx_t _ISet_m(put)(_ISet_obj_t iset, _ISet_key_t key);

// Hash Set remove
_ISet_key_t * _ISet_m(rem)(_ISet_obj_t iset, _ISet_idx_t idx, _ISet_key_t * key);

// Apply callback to all entries, in slot order
_ISet_size_t _ISet_m(visit)(_ISet_obj_t iset, _ISet_vis_t visitor, void * dat);




//###
//###  END
//###
//################################################################################################
#endif //__ISET__BODY__


