#ifndef __LIST__HEADER__
#define __LIST__HEADER__
//################################################################################################
//##################                                                            ##################
//##################                Indexed List Datastructure                  ##################
//##################                                                            ##################
//##################                        Header File                         ##################
//##################                                                            ##################
//################################################################################################
///
#endif // __LIST__HEADER__

#ifndef __LIST__LICENCE__
#define __LIST__LICENCE__
//
// List: High-efficiency Indexed List object model.
//
// Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
//
//
// This library is free software;  you can redistribute it and/or modify it
// under  the terms of the  GNU Lesser General Public License  as published
// by the  Free Software Foundation;  either version 2.1 of the License, or
// (at your option) any later version.
//
// This  library  is  distributed  in  the hope that it will be useful, but
// WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
// General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library;  if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
//
// Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
// Contact: varkhan@varkhan.net
// Website: http://www.varkhan.net/
// Project: http://www.varkhan.net/software/VCom
///
#endif // __LIST__LICENCE__



#ifndef __LIST__BODY__
#define __LIST__BODY__



#ifndef __LIST__TYPEDEF_List_vis_t
#define __LIST__TYPEDEF_List_vis_t
// The prototype of visitor callbacks
typedef _List_size_t (* _List_vis_t)(_List_idx_t idx, _List_val_t * val, void * dat);
#endif



//################################################################################################
//###
//###  Structures and types
//###



// The opaque List object definition
typedef struct _List_struct_t * _List_obj_t;


//################################################################################################
//###
//###  Object lifecycle
//###


// Builds a new list in memory
_List_obj_t _List_m(new)(_List_size_t blen, _List_size_t capa);

// Copies a list in another memory segment
_List_obj_t _List_m(cpy)(_List_obj_t list);

// Deletes a list from memory
void _List_m(del)(_List_obj_t list);



//################################################################################################
//###
//###  Parameters accessors
//###


// The number of elements in the list
_List_size_t _List_m(size)(_List_obj_t list);

// The head index (highest unused index) in the list
_List_idx_t _List_m(head)(_List_obj_t list);

// The tail index (lowest unused index) in the list
_List_idx_t _List_m(tail)(_List_obj_t list);

// The capacity (total numer of slots) in the list
_List_size_t _List_m(capa)(_List_obj_t list);




//################################################################################################
//###
//###  Bulk operations
//###


// Clears all elements
void _List_m(clr)(_List_obj_t list);

// Release unused ressources
void _List_m(adj)(_List_obj_t list, _List_size_t capa);



//################################################################################################
//###
//###  Elementary operations
//###


// Tests the presence of an element in a slot
_List_bool_t _List_m(has)(_List_obj_t list, _List_idx_t idx);

// Retries an element from a slot
_List_val_t * _List_m(get)(_List_obj_t list, _List_idx_t idx);

// Updates the contents of a slot
_List_val_t * _List_m(set)(_List_obj_t list, _List_idx_t idx, _List_val_t val);

// Removes (empties) a slot from its element
_List_val_t * _List_m(rem)(_List_obj_t list, _List_idx_t idx, _List_val_t * val);

// Apply a callback to all entries, in slot order
_List_size_t _List_m(visit)(_List_obj_t list, _List_vis_t visitor, void * dat);




//###
//###  END
//###
//################################################################################################
#endif //__LIST__BODY__


