//################################################################################################
//##################                                                            ##################
//##################                   Bit manipulation tools                   ##################
//##################                                                            ##################
//##################                      Implementation                        ##################
//##################                                                            ##################
//################################################################################################


/*
 * Bits: High-efficiency Bit manipulation tools.
 *
 * Copyright (C) 2006-2009 Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:  Yann LANDRIN-SCHWEITZER, a.k.a. Varkhan
 * Contact: varkhan@varkhan.net
 * Website: http://www.varkhan.net/
 * Project: http://www.varkhan.net/software/VCom
 */


// Include header
#include "_Bits_.h"


//################################################################################################
//###
//###  Bit counts and positions
//###


// Table of number of set bits in a character
static const unsigned char __Bits_ctbit_table[] = {
	0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
};


// Table of number of trailing 0-bits in a character
static const unsigned char __Bits_lobit_table[] = {
	8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
	5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
};


// Table of number of leading 0-bits in a character
static const unsigned char __Bits_hibit_table[] = {
	8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
};




#if defined __alpha_cix__
# define __Bits_ctbit(x)						\
	({ unsigned long __btr;						\
	__asm__("ctpop %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
# define __Bits_lobit(x)						\
	({ unsigned long __btr;						\
	__asm__("cttz %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
# define __Bits_hibit(x)						\
	({ unsigned long __btr;						\
	__asm__("ctlz %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
#elif defined(__GNUC__) && defined (__i386__)
/*# define __Bits_ctbit(x)						\
	({ unsigned long __btr;						\
	__asm__("popcountl %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; }) */
# define __Bits_lobit(x)						\
	({ unsigned long __btr;						\
	__asm__("bsfl %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
# define __Bits_hibit(x)						\
	({ unsigned long __btr;						\
	__asm__("bsrl %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
#elif defined(__GNUC__) && defined(__x86_64__)
/*# define __Bits_ctbit(x)						\
	({ unsigned long __btr;						\
	__asm__("popcountq %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; }) */
# define __Bits_lobit(x)						\
	({ unsigned long __btr;						\
	__asm__("bsfq %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
# define __Bits_hibit(x)						\
	({ unsigned long __btr;						\
	__asm__("bsrq %1,%0" : "=r"(__btr) : "r"(x));			\
	__btr; })
//#else//if __GNUC__ == 3 && __GNUC_MINOR__ >= 4 || __GNUC__ > 3
//# define __Bits_ctbit(x)	__builtin_popcountl(x)
//# define __Bits_lobit(x)	__builtin_ctzl(x)
//# define __Bits_hibit(x)	__builtin_clzl(x)
#endif


//########################################
//# Number of set bits

_Bits_size_t _Bits_b(ctbit)(_Bits_byte_t bits) {
#ifdef __Bits_ctbit
	return __Bits_ctbit((_Bits_long_t)bits);
#else
	return	__Bits_ctbit_table[bits];
#endif
}

_Bits_size_t _Bits_l(ctbit)(_Bits_long_t bits) {
#ifdef __Bits_ctbit
	return __Bits_ctbit(bits);
#else
	return	__Bits_ctbit_table[ bits     &0xFF]
	  +	__Bits_ctbit_table[(bits>> 8)&0xFF]
	  +	__Bits_ctbit_table[(bits>>16)&0xFF]
	  +	__Bits_ctbit_table[(bits>>24)&0xFF]
	  ;
#endif
}

_Bits_size_t _Bits_v(ctbit)(void * bits, _Bits_size_t size) {
	_Bits_size_t s=0;
	_Bits_size_t i=0;
#ifdef __Bits_ctbit
	_Bits_size_t j=sizeof(unsigned long);
	while( (j=i+sizeof(unsigned long)) <= size) {
		s += __Bits_ctbit(* (unsigned long *) (bits+i));
		i = j;
	}
	while(i<size) {
		s += __Bits_ctbit(* (unsigned char *) (bits+i));
		i ++;
	}
#else
	while(i<size) {
		s += __Bits_ctbit_table[* (unsigned char *) (bits+i)];
		i++;
	}
#endif
	return s;
}


//########################################
//# Number of trailing 0-bits, starting at the least significant bit position

_Bits_size_t _Bits_b(lobit)(_Bits_byte_t bits) {
#ifdef __Bits_lobit
	if(bits) return __Bits_lobit((_Bits_long_t)bits);
#else
	if(bits) return __Bits_lobit_table[bits];
#endif
	return -1L;
}

_Bits_size_t _Bits_l(lobit)(_Bits_long_t bits) {
#ifdef __Bits_lobit
	if(bits) return __Bits_lobit(bits);
#else
	if(bits&0x000000FF)	return    __Bits_lobit_table[ bits     &0xFF];
	if(bits&0x0000FFFF)	return  8+__Bits_lobit_table[(bits>> 8)&0xFF];
	if(bits&0x00FFFFFF)	return 16+__Bits_lobit_table[(bits>>16)&0xFF];
	if(bits)		return 24+__Bits_lobit_table[(bits>>24)&0xFF];
#endif
	return -1L;
}

_Bits_size_t _Bits_v(lobit)(void * bits, _Bits_size_t size) {
	_Bits_size_t i = 0;
#ifdef __Bits_lobit
	_Bits_size_t j;
	while( (j=i+sizeof(unsigned long)) <= size ) {
		unsigned long b = * (unsigned long *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit(b);
		i = j;
	}
	while(i<size) {
		unsigned long b = * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit(b);
		i ++;
	}
#else
	while(i<size) {
		unsigned char b = * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit_table[b];
		i ++;
	}
#endif
	return -1L;
}

_Bits_size_t _Bits_v(lobit1)(void * bits, _Bits_size_t size) {
	_Bits_size_t i = 0;
#ifdef __Bits_lobit
	_Bits_size_t j;
	while( (j=i+sizeof(unsigned long)) <= size ) {
		unsigned long b = ~ * (unsigned long *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit(b);
		i = j;
	}
	while(i<size) {
		unsigned long b = ~ * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit(b);
		i ++;
	}
#else
	while(i<size) {
		unsigned char b = ~ * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_lobit_table[b];
		i ++;
	}
#endif
	return -1L;
}


//########################################
// Number of leading 0-bits, starting at the most significant bit position

_Bits_size_t _Bits_b(hibit)(_Bits_byte_t bits) {
#ifdef __Bits_hibit
	if(bits) return __Bits_hibit((_Bits_long_t)bits);
#else
	if(bits) return  __Bits_hibit_table[bits];
#endif
	return -1L;
}

_Bits_size_t _Bits_l(hibit)(_Bits_long_t bits) {
#ifdef __Bits_hibit
	if(bits) return __Bits_hibit(bits);
#else
	if(bits&0xFF000000)	return    __Bits_hibit_table[(bits>>24)&0xFF];
	if(bits&0x00FF0000)	return  8+__Bits_hibit_table[(bits>>16)&0xFF];
	if(bits&0x0000FF00)	return 16+__Bits_hibit_table[(bits>> 8)&0xFF];
	if(bits)		return 24+__Bits_hibit_table[ bits     &0xFF];
#endif
	return -1L;
}

_Bits_size_t _Bits_v(hibit)(void * bits, _Bits_size_t size) {
	_Bits_size_t i=size;
#ifdef __Bits_hibit
	while( i >= sizeof(unsigned long) ) {
		i -= sizeof(unsigned long);
		unsigned long b = * (unsigned long *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit(b);
	}
	while(i>0) {
		i --;
		unsigned long b = * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit(b);
	}
#else
	while(i>0) {
		i --;
		unsigned char b = * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit_table[b];
	}
#endif
	return -1L;
}

_Bits_size_t _Bits_v(hibit1)(void * bits, _Bits_size_t size) {
	_Bits_size_t i=size;
#ifdef __Bits_hibit
	while( i >= sizeof(unsigned long) ) {
		i -= sizeof(unsigned long);
		unsigned long b = ~ * (unsigned long *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit(b);
	}
	while(i>0) {
		i --;
		unsigned long b = ~ * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit(b);
	}
#else
	while(i>0) {
		i --;
		unsigned char b = ~ * (unsigned char *) (bits+i);
		if(b) return (i<<3) + __Bits_hibit_table[b];
	}
#endif
	return -1L;
}




//###
//###  END
//###
//################################################################################


